#!/home/ywan/soft/anaconda3/bin/python
import matplotlib.pyplot as plt
import numpy as np
from sklearn.model_selection import train_test_split, ShuffleSplit, learning_curve
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

def frame(num):
    bx=plt.gca()
    bx.spines['bottom'].set_linewidth(num)
    bx.spines['left'].set_linewidth(num)
    bx.spines['right'].set_linewidth(num)
    bx.spines['top'].set_linewidth(num)

def loadDataSet(fileName):
    fr = open(fileName)
    dataMat = []
    for line in fr.readlines():
        lineArr = []
        curLine = line.split()
        for i in range(len(curLine)):
            lineArr.append(float(curLine[i]))
        dataMat.append(lineArr)
    dataMat = np.array(dataMat)
    return dataMat

def drawbar_co(names,a,b,c,d,e,f):
    font={'weight':'regular', 'color':'black', 'size':12}
    name = ['HOMO$_D$','LUMO$_D$','HOMO$_A$','LUMO$_A$','L',r'$\chi$-','CN',r'$\chi$+']
    indey = np.arange(-1.0, 1.01, 0.4)
    plt.figure(figsize=(12,3.6),dpi=(300))
    plt.subplot(1,3,1)
    plt.bar(names, a, color='deepskyblue',label='$\itp$')
    plt.plot(names,d,'o-',label='$\itQ$')
    plt.yticks(indey,size=12)
    plt.xticks(names,name,rotation=270)
    frame(1.0)
    plt.legend(loc="best",fontsize=12)
    plt.subplot(1,3,2)
    plt.bar(names, b, color='tomato',label='$\itp$')
    plt.plot(names,e,'o-',label='$\itQ$')
    plt.yticks(indey,size=12)
    plt.xticks(names,name,rotation=270)
    frame(1.0)
    plt.legend(loc="best",fontsize=12)
    plt.subplot(1,3,3)
    plt.bar(names, c, color='limegreen',label='$\itp$')
    plt.plot(names,f,'o-',label='$\itQ$')
    plt.yticks(indey,size=12)
    plt.xticks(names,name,rotation=270)
    frame(1.0)
    plt.legend(loc="best",fontsize=12)
    plt.subplots_adjust(wspace=0.3, hspace=None)
    plt.savefig('feature_select.png',bbox_inches='tight')
    #plt.show()

def drawHist(X,num_bins,name):
    plt.figure(figsize=(6,6),dpi=300)
    plt.subplot(1,1,1)
    plt.hist(X, num_bins, facecolor='grey')
    font={'weight':'regular', 'color':'black', 'size':18}
    plt.xlabel('error of %s (eV)' % name, font)
    plt.ylabel('frequency', font)
    index = np.arange(-0.12,0.121,0.04)
    plt.xticks(index,size=14)
    plt.xticks(size=14)
    plt.yticks(size=16)
    frame(2.5)
    plt.subplots_adjust(left=0.15)
    plt.savefig('range_of_%s.png' % name,bbox_inches='tight')
#    plt.show()


def drawline(X,y):
    plt.figure()
    plt.plot(X,y)
    plt.xlabel('Range')
    plt.ylabel('Probability')
    plt.subplots_adjust(left=0.15)
    plt.show()

def plot_learning_curve(estimator, title, X, y, ylim=None, cv=None, n_jobs=1, train_sizes=np.linspace(.2, 1.0, 8)):
    plt.figure()
    font={'weight':'regular', 'color':'black', 'size':16}
    plt.title(title,size=10)
    if ylim is not None:
        plt.ylim(*ylim)
    plt.xlabel("Training examples",font)
    plt.ylabel("Score",font)
    plt.xticks(size=14)
    plt.yticks(size=14)
    train_sizes,train_scores,test_scores=learning_curve(estimator,X,y,cv=cv,n_jobs=n_jobs,train_sizes=train_sizes)
    t_scores_mean = np.mean(train_scores, axis=1)
    t_scores_std = np.std(train_scores, axis=1)
    test_scores_mean = np.mean(test_scores, axis=1)
    test_scores_std = np.std(test_scores, axis=1)
    plt.grid()
    plt.fill_between(train_sizes,t_scores_mean-t_scores_std,t_scores_mean+t_scores_std,alpha=0.1,color='r')
    plt.fill_between(train_sizes,test_scores_mean-test_scores_std,test_scores_mean+test_scores_std, alpha=0.1, color="g")
    plt.plot(train_sizes, t_scores_mean, 'o-', color="r",label="Training score")
    plt.plot(train_sizes, test_scores_mean, 'o-', color="g", label="Cross-validation score")
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(2)
    ax.spines['left'].set_linewidth(2)
    ax.spines['right'].set_linewidth(2)
    ax.spines['top'].set_linewidth(2)
    plt.legend(loc="best",fontsize=14)
    return plt

def comparison(a,b,c,d,e,f,target):
    xmajorFormatter = FormatStrFormatter('%.1f')
    plt.figure(figsize=(6,6),dpi=300)
    ax = plt.subplot(1,1,1)
    font={'weight':'regular', 'color':'black', 'size':18}
    plt.ylabel("%s (eV)" % target, font)
    #plt.ylabel("%s by Machine Learning (eV)" % target,font)
    #plt.scatter(a,b, marker='o', label='train')
    plt.scatter(c,d, marker='o', label='test')
    plt.scatter(e,f, marker='o', label='verification')
    plt.legend(loc="best",fontsize=18)
    ax.xaxis.set_major_formatter(xmajorFormatter)
    ax.yaxis.set_major_formatter(xmajorFormatter)
    ALL = []
    for m in [a,b,c,d,e,f]:
        for i in m:
            ALL.append(i)
    x = np.arange(min(ALL),max(ALL),0.01)
    y = x
    plt.plot(x,y,':',c='black')
    #index = np.arange(round(min(ALL),1),round(max(ALL),1),0.3)
    plt.xticks(size=18)
    plt.yticks(size=18)
    frame(2.0)
    plt.grid()
    plt.savefig('%s_comparison' % target,bbox_inches='tight')
#    plt.show()

def statistics(datalist):
    dic = {'A':0,'B':0,'C':0,'D':0,'E':0,'F':0,'G':0,'H':0,'I':0,'J':0}
    for x in datalist:
        if x<=-0.4:
            dic['A']+=1
        elif -0.4<x<=-0.3:
            dic['B']+=1
        elif -0.3<x<=-0.2:
            dic['C']+=1
        elif -0.2<x<=-0.1:
            dic['D']+=1
        elif -0.1<x<=0.0:
            dic['E']+=1
        elif 0.0<x<=0.1:
            dic['F']+=1
        elif 0.1<x<=0.2:
            dic['G']+=1
        elif 0.2<x<=0.3:
            dic['H']+=1
        elif 0.3<x<=0.4:
            dic['I']+=1
        else:
            dic['J']+=1
    statist = []
    for i in ['A','B','C','D','E','F','G','H','I','J']:
        statist.append(dic[i])
    return statist

def error(datalist,N,target):
    plt.figure()
    index = np.arange(N)
    width = 0.30
    p2 = plt.bar(index, datalist, width, label="relative error %", color=['r','orange','gold','yellowgreen','g','g','yellowgreen','gold','orange','r'])
    font={'weight':'bold', 'color':'black', 'size':14}
    plt.xlabel('%s error range (eV)' % target, font)
    plt.ylabel('Frequency', font)
    plt.xticks(index, ['<-0.4','-0.4~-0.3','-0.3~-0.2','-0.2~-0.1','-0.1~-0.0','0.0~0.1','0.1~0.2','0.2~0.3','0.3~0.4','>0.4'] ,size='8')
    plt.yticks(np.arange(0,120,10),size='12')
    ax=plt.gca();
    ax.spines['bottom'].set_linewidth(1.5)
    ax.spines['left'].set_linewidth(1.5)
    ax.spines['right'].set_linewidth(1.5)
    ax.spines['top'].set_linewidth(1.5)
    plt.savefig('%s_error.png' % target)
    plt.show()

def score():
    gap = np.loadtxt('svr-gap-score.dat' % name)
    IP = np.loadtxt('svr-IP-score.dat' % name)
    x = np.arange(1,10.1,1)
    xmajorFormatter = FormatStrFormatter('%.4f')
    dic = {0:'train',1:'test'}
    plt.figure(figsize=(14,7),dpi=300)
    ax = plt.subplot(1,3,1)
    for i in range(2):
        plt.plot(x,gap[:,i],'-o',label=dic[i])
    frame(1)
    plt.legend(loc="best",fontsize=10)
    plt.ylabel('MAPE_SVR')
    plt.xlabel('times')
    plt.xticks(x,size='10')
    ax.yaxis.set_major_formatter(xmajorFormatter)
    ax = plt.subplot(1,3,2)
    for i in range(2):
        plt.plot(x,svr[:,2+i],'-o',label=dic[i])
    frame(1)
    plt.legend(loc="best",fontsize=10)
    plt.ylabel('RMSE_SVR')
    plt.xlabel('times')
    plt.xticks(x,size='10')
    ax.yaxis.set_major_formatter(xmajorFormatter)
    ax = plt.subplot(1,3,3)
    for i in range(2):
        plt.plot(x,svr[:,4+i],'-o',label=dic[i])
    frame(1)
    plt.legend(loc="best",fontsize=10)
    plt.ylabel('R2_SVR')
    plt.xlabel('times')
    plt.xticks(x,size='10')
    ax.yaxis.set_major_formatter(xmajorFormatter)
    plt.subplots_adjust(left=None, bottom=None, right=None, top=None,wspace=0.3, hspace=None)
    plt.savefig('score-svr.png',bbox_inches='tight')
#    plt.show()
